using System;
using System.Text;
using Ceix30ClassLib;
using Ceix30ClassLib.Arinc;
namespace Arinc429Example
{
	public class ceix30_example
	{
		static void Main(string[] args)
		{
			short board = 0;             // Device Number fixed as Board 0
			short channelA = 0;          // Demo using Transmit/Receive Channel pair 0
         short channelB = 1;          // Demo using Transmit/Receive Channel pair 1
			int receivedMessage = 0;     // Received ARINC 429 message storage
         Int64 timeStamp = 0;         // 64-bit time-stamp used with ar_get_latest_t

         // allocate storage for the time-tag structure used with ar_getnext_xt
			Arinc.AR_TIMETAG_TYPE timeTagStruct = new Arinc.AR_TIMETAG_TYPE();

         // allocate storage for the scheduled transmit message structure array used with ar_define_msg_block
         Arinc.AR_SCHEDULED_MSG_ENTRY_TYPE[] scheduledMessageSet = new Arinc.AR_SCHEDULED_MSG_ENTRY_TYPE[] {
            new Arinc.AR_SCHEDULED_MSG_ENTRY_TYPE(),
            new Arinc.AR_SCHEDULED_MSG_ENTRY_TYPE()
         };

         string apiVersion = "        ";       // API Version
         Int16 fwVersion = 0;                  // Board Firmware Version
         Int32 fwVersionMajor = 0;             // Board Firmware Version Major 
         Int32 fwVersionMinor = 0;             // Board Firmware Version Minor

         Arinc.ar_version(ref apiVersion);
         Console.WriteLine("CEI-x30 API Version " + apiVersion);

			// open a session with the CEI-x30 board
			short status = Arinc.ar_open (board);
			if (status == Arinc.ARS_NORMAL) 
			{
            // acquire and parse the firmware version for reporting on the console
            status = Arinc.ar_get_device_config(board, 0, Arinc.ARU_FW_VERSION, ref fwVersion);
            fwVersionMajor = fwVersion;
            fwVersionMinor = fwVersion;
            fwVersionMajor = (fwVersionMajor & 0x00F0) >> 4;
            fwVersionMinor = (fwVersionMinor & 0x000F);
            Console.WriteLine("CEI-x30 Board " + board.ToString() + " with f/w ver. " + fwVersionMajor.ToString("X1") + "." + fwVersionMinor.ToString("X1") + " initialization successful.");

            // execute an internal wrap self-test
            status = Arinc.ar_board_test(board, Arinc.INTERNAL_WRAP);
      		Console.WriteLine("SELF TEST: " + status.ToString());

				// create some periodic messages at a 20Hz repeat rate, one on each for the first two channels
            for (int messageIndex = 0; messageIndex < 2; messageIndex++)
            {
               scheduledMessageSet[messageIndex].board = board;
               scheduledMessageSet[messageIndex].rate = 50;
               scheduledMessageSet[messageIndex].start = 0;
               scheduledMessageSet[messageIndex].txCount = Arinc.ARU_SCHED_MSG_INFINITE;
            }
            scheduledMessageSet[0].channel = (int)channelA;
            scheduledMessageSet[0].data = 0x01110101;
            scheduledMessageSet[1].channel = (int)channelB;
            scheduledMessageSet[1].data = 0x02220202;

            // now define the periodic messages on the board
            status = Arinc.ar_define_msg_block(2, scheduledMessageSet);

				// enable reception with internal wrap, default time-tag mode
            status = Arinc.ar_set_device_config(board, channelA, Arinc.ARU_RECV_MODE, Arinc.AR_WRAP_ON);
            status = Arinc.ar_set_device_config(board, channelB, Arinc.ARU_RECV_MODE, Arinc.AR_WRAP_ON);
            status = Arinc.ar_set_device_config(board, 0, Arinc.ARU_RX_TIMETAG_MODE, Arinc.AR_TIMETAG_INT_USEC_64BIT);

				// enable all board message processing 
				status = Arinc.ar_go (board);

            // initialize the time tag format in the 64-bit time-tag structure for ar_getnext_xt to use a 1 microsecond resolution time-stamp
            timeTagStruct.timeTagFormat = (Int64)(Arinc.AR_TIMETAG_INT_USEC_64BIT);

            // attempt to receive up to 10 messages from the receive channel 0 circular buffer, then terminate.
				for (int messageIndex = 0; messageIndex < 10; messageIndex++) 
				{
					// invoke ar_getnext_xt to delay up to 1/2 second for the next circular buffered message on Rx 0
               status = Arinc.ar_getnext_xt(board, channelA, ref receivedMessage, ref timeTagStruct);
					
					// if a message was received, display the message and time-stamp
					if (status == Arinc.ARS_GOTDATA) 
					{
                  Console.WriteLine("Channel " + channelA.ToString() + " buffered message: $" + receivedMessage.ToString("X8") + " time-stamp: " + timeTagStruct.timeTag.ToString() + " microseconds");
               }

               // invoke ar_get_latest_t to query the snapshot buffer for the latest received message for label o002 on Rx 1
               int status2 = Arinc.ar_get_latest_t(board, channelB, 2, ref receivedMessage, ref timeStamp);

               // if a message was detected from the snapshot buffer (message value > 0), display the message and 64-bit time-stamp
               if ((status2 == Arinc.ARS_NORMAL) && (receivedMessage != 0))
               {
                  Console.WriteLine("Channel " + channelB.ToString() + " snapshot message: $" + receivedMessage.ToString("X8") + " time-stamp: " + timeStamp.ToString() + " microseconds");
               }
            }
   
				// disable all board processing 
				status = Arinc.ar_stop(board);
			}
			else
			{
				Console.WriteLine("CEI-x30 Board " + board.ToString() + " initialization failed with status code: " + status.ToString());
			}

			// terminate the session with the CEI-x30 board
			status = Arinc.ar_close(board);

         Console.WriteLine("Press any key to terminate.");
         Console.ReadKey(false);
		}
	}
}
